@@ -35,12 +35,65 @@
 #include <stdio.h>
 
 #include <vips/vips.h>
-#include <vips/deprecated.h>
+#include <vips/internal.h>
 
 #ifdef WITH_DMALLOC
 #include <dmalloc.h>
 #endif /*WITH_DMALLOC*/
 
+/* One image in, one out.
+ */
+static im_arg_desc one_in_one_out[] = {
+	IM_INPUT_IMAGE( "in" ),
+	IM_OUTPUT_IMAGE( "out" )
+};
+
+/* Two images in, one out.
+ */
+static im_arg_desc two_in_one_out[] = {
+	IM_INPUT_IMAGE( "in1" ),
+	IM_INPUT_IMAGE( "in2" ),
+	IM_OUTPUT_IMAGE( "out" )
+};
+
+/* Call im_clip via arg vector.
+ */
+static int
+clip_vec( im_object *argv )
+{
+	return( im_clip( argv[0], argv[1] ) );
+}
+
+/* Description of im_clip.
+ */
+static im_function clip_desc = {
+	"im_clip", 			/* Name */
+	"convert to unsigned 8-bit integer",
+	IM_FN_PTOP | IM_FN_PIO,		/* Flags */
+	clip_vec, 			/* Dispatch function */
+	IM_NUMBER( one_in_one_out ), 	/* Size of arg list */
+	one_in_one_out 			/* Arg list */
+};
+
+/* Call im_c2ps via arg vector.
+ */
+static int
+c2ps_vec( im_object *argv )
+{
+	return( im_c2ps( argv[0], argv[1] ) );
+}
+
+/* Description of im_c2ps.
+ */
+static im_function c2ps_desc = {
+	"im_c2ps", 			/* Name */
+	"find power spectrum of complex image",
+	IM_FN_PTOP | IM_FN_PIO,		/* Flags */
+	c2ps_vec, 			/* Dispatch function */
+	IM_NUMBER( one_in_one_out ), 	/* Size of arg list */
+	one_in_one_out 			/* Arg list */
+};
+
 /* Args for im_lhisteq.
  */
 static im_arg_desc lhisteq_args[] = {
@@ -97,21 +150,6 @@ static im_arg_desc conv_dmask[] = {
 	IM_INPUT_DMASK( "matrix" )
 };
 
-/* One image in, one out.
- */
-static im_arg_desc one_in_one_out[] = {
-	IM_INPUT_IMAGE( "in" ),
-	IM_OUTPUT_IMAGE( "out" )
-};
-
-/* Two images in, one out.
- */
-static im_arg_desc two_in_one_out[] = {
-	IM_INPUT_IMAGE( "in1" ),
-	IM_INPUT_IMAGE( "in2" ),
-	IM_OUTPUT_IMAGE( "out" )
-};
-
 /* Call im_cmulnorm via arg vector.
  */
 static int
@@ -1180,9 +1218,199 @@ static im_function convf_desc = {
 	conv_dmask 			/* Arg list */
 };
 
+/* Args for im_circle.
+ */
+static im_arg_desc circle_args[] = {
+	IM_RW_IMAGE( "image" ),
+	IM_INPUT_INT( "cx" ),
+	IM_INPUT_INT( "cy" ),
+	IM_INPUT_INT( "radius" ),
+	IM_INPUT_INT( "intensity" )
+};
+
+/* Call im_circle via arg vector.
+ */
+static int
+circle_vec( im_object *argv )
+{
+	int cx = *((int *) argv[1]);
+	int cy = *((int *) argv[2]);
+	int radius = *((int *) argv[3]);
+	int intensity = *((int *) argv[4]);
+
+	return( im_circle( argv[0], cx, cy, radius, intensity ) );
+}
+
+/* Description of im_circle.
+ */ 
+static im_function circle_desc = {
+	"im_circle", 			/* Name */
+	"plot circle on image",
+	0,				/* Flags */
+	circle_vec, 			/* Dispatch function */
+	IM_NUMBER( circle_args ), 	/* Size of arg list */
+	circle_args 			/* Arg list */
+};
+
+/* Args for im_flood_blob_copy().
+ */
+static im_arg_desc flood_blob_copy_args[] = {
+	IM_INPUT_IMAGE( "in" ),
+	IM_OUTPUT_IMAGE( "out" ),
+	IM_INPUT_INT( "start_x" ),
+	IM_INPUT_INT( "start_y" ),
+	IM_INPUT_DOUBLEVEC( "ink" )
+};
+
+/* Call im_flood_blob_copy() via arg vector.
+ */
+static int
+flood_blob_copy_vec( im_object *argv )
+{
+	IMAGE *in = argv[0];
+	IMAGE *out = argv[1];
+	int start_x = *((int *) argv[2]);
+	int start_y = *((int *) argv[3]);
+	im_doublevec_object *dv = (im_doublevec_object *) argv[4];
+
+	PEL *ink;
+
+	if( !(ink = im__vector_to_ink( "im_flood_blob_copy",
+		in, dv->n, dv->vec )) )
+		return( -1 );
+
+	return( im_flood_blob_copy( in, out, start_x, start_y, ink ) );
+}
+
+/* Description of im_flood_blob_copy().
+ */ 
+static im_function flood_blob_copy_desc = {
+	"im_flood_blob_copy",	/* Name */
+	"flood with ink from start_x, start_y while pixel == start pixel",
+	0,			/* Flags */
+	flood_blob_copy_vec, 	/* Dispatch function */
+	IM_NUMBER( flood_blob_copy_args ),/* Size of arg list */
+	flood_blob_copy_args 	/* Arg list */
+};
+
+/* Args for im_flood_copy().
+ */
+static im_arg_desc flood_copy_args[] = {
+	IM_INPUT_IMAGE( "in" ),
+	IM_OUTPUT_IMAGE( "out" ),
+	IM_INPUT_INT( "start_x" ),
+	IM_INPUT_INT( "start_y" ),
+	IM_INPUT_DOUBLEVEC( "ink" )
+};
+
+/* Call im_flood_copy() via arg vector.
+ */
+static int
+flood_copy_vec( im_object *argv )
+{
+	IMAGE *in = argv[0];
+	IMAGE *out = argv[1];
+	int start_x = *((int *) argv[2]);
+	int start_y = *((int *) argv[3]);
+	im_doublevec_object *dv = (im_doublevec_object *) argv[4];
+
+	PEL *ink;
+
+	if( !(ink = im__vector_to_ink( "im_flood_copy",
+		in, dv->n, dv->vec )) )
+		return( -1 );
+
+	return( im_flood_copy( in, out, start_x, start_y, ink ) );
+}
+
+/* Description of im_flood_copy().
+ */ 
+static im_function flood_copy_desc = {
+	"im_flood_copy",	/* Name */
+	"flood with ink from start_x, start_y while pixel == start pixel",
+	0,			/* Flags */
+	flood_copy_vec, 	/* Dispatch function */
+	IM_NUMBER( flood_copy_args ),/* Size of arg list */
+	flood_copy_args 	/* Arg list */
+};
+
+/* Args for im_flood_other_copy().
+ */
+static im_arg_desc flood_other_copy_args[] = {
+	IM_INPUT_IMAGE( "test" ),
+	IM_INPUT_IMAGE( "mark" ),
+	IM_OUTPUT_IMAGE( "out" ),
+	IM_INPUT_INT( "start_x" ),
+	IM_INPUT_INT( "start_y" ),
+	IM_INPUT_INT( "serial" )
+};
+
+/* Call im_flood_other_copy() via arg vector.
+ */
+static int
+flood_other_copy_vec( im_object *argv )
+{
+	IMAGE *test = argv[0];
+	IMAGE *mark = argv[1];
+	IMAGE *out = argv[2];
+	int start_x = *((int *) argv[3]);
+	int start_y = *((int *) argv[4]);
+	int serial = *((int *) argv[5]);
+
+	return( im_flood_other_copy( test, mark, out, 
+		start_x, start_y, serial ) );
+}
+
+/* Description of im_flood_other_copy().
+ */ 
+static im_function flood_other_copy_desc = {
+	"im_flood_other_copy",	/* Name */
+	"flood mark with serial from start_x, start_y while pixel == start pixel",
+	0,			/* Flags */
+	flood_other_copy_vec, 	/* Dispatch function */
+	IM_NUMBER( flood_other_copy_args ),/* Size of arg list */
+	flood_other_copy_args 	/* Arg list */
+};
+
+/* Args for im_insertplace.
+ */
+static im_arg_desc insertplace_args[] = {
+	IM_RW_IMAGE( "main" ),
+	IM_INPUT_IMAGE( "sub" ),
+	IM_INPUT_INT( "x" ),
+	IM_INPUT_INT( "y" )
+};
+
+/* Call im_insertplace via arg vector.
+ */
+static int
+insertplace_vec( im_object *argv )
+{
+	int x = *((int *) argv[2]);
+	int y = *((int *) argv[3]);
+
+	return( im_insertplace( argv[0], argv[1], x, y ) );
+}
+
+/* Description of im_insertplace.
+ */ 
+static im_function insertplace_desc = {
+	"im_insertplace", 		/* Name */
+	"draw image sub inside image main at position (x,y)",
+	0,				/* Flags */
+	insertplace_vec, 		/* Dispatch function */
+	IM_NUMBER( insertplace_args ), 	/* Size of arg list */
+	insertplace_args 		/* Arg list */
+};
+
 /* Package up all these functions.
  */
 static im_function *deprecated_list[] = {
+	&flood_copy_desc,
+	&flood_blob_copy_desc,
+	&flood_other_copy_desc,
+	&clip_desc,
+	&c2ps_desc,
 	&resize_linear_desc,
 	&cmulnorm_desc,
 	&fav4_desc,
@@ -1223,7 +1451,9 @@ static im_function *deprecated_list[] = {
 	&dilate_raw_desc,
 	&erode_raw_desc,
 	&similarity_area_desc,
-	&similarity_desc
+	&similarity_desc,
+	&insertplace_desc,
+	&circle_desc
 };
 
 /* Package of functions.
