@@ -29,6 +29,15 @@
  *	- fix signed/unsigned warning
  * 13/8/09
  * 	- allow "none" for profile, meaning don't embed one
+ * 4/2/10
+ * 	- gtkdoc
+ * 17/7/10
+ * 	- use g_assert()
+ * 	- allow space for the header in init_destination(), helps writing very
+ * 	  small JPEGs (thanks Tim Elliott)
+ * 18/7/10
+ * 	- collect im_vips2bufjpeg() output in a list of blocks ... we no
+ * 	  longer overallocate or underallocate
  */
 
 /*
@@ -58,8 +67,8 @@
  */
 
 /*
-#define DEBUG
 #define DEBUG_VERBOSE
+#define DEBUG
  */
 
 #ifdef HAVE_CONFIG_H
@@ -104,7 +113,6 @@ im_vips2mimejpeg( IMAGE *in, int qfac )
 #include <stdlib.h>
 #include <string.h>
 #include <setjmp.h>
-#include <assert.h>
 
 #ifdef HAVE_EXIF
 #ifdef UNTAGGED_EXIF
@@ -199,7 +207,6 @@ typedef struct {
 	IMAGE *in;
 	struct jpeg_compress_struct cinfo;
         ErrorManager eman;
-	im_threadgroup_t *tg;
 	JSAMPROW *row_pointer;
 	char *profile_bytes;
 	unsigned int profile_length;
@@ -210,7 +217,6 @@ static void
 write_destroy( Write *write )
 {
 	jpeg_destroy_compress( &write->cinfo );
-	IM_FREEF( im_threadgroup_free, write->tg );
 	IM_FREEF( im_close, write->in );
 	IM_FREEF( fclose, write->eman.fp );
 	IM_FREE( write->row_pointer );
@@ -219,6 +225,15 @@ write_destroy( Write *write )
 	im_free( write );
 }
 
+#define UC IM_BANDFMT_UCHAR
+
+/* Type promotion for save ... just always go to uchar.
+ */
+static int bandfmt_jpeg[10] = {
+/* UC  C   US  S   UI  I   F   X   D   DX */
+   UC, UC, UC, UC, UC, UC, UC, UC, UC, UC
+};
+
 static Write *
 write_new( IMAGE *in )
 {
@@ -228,14 +243,13 @@ write_new( IMAGE *in )
 		return( NULL );
 	memset( write, 0, sizeof( Write ) );
 
-	if( !(write->in = im__convert_saveable( in, IM__RGB_CMYK )) ) {
+	if( !(write->in = im__convert_saveable( in, 
+		IM__RGB_CMYK, bandfmt_jpeg )) ) {
 		im_error( "im_vips2jpeg", 
 			"%s", _( "unable to convert to saveable format" ) );
 		write_destroy( write );
 		return( NULL );
-	}
-
-	write->tg = NULL;
+	} 
 	write->row_pointer = NULL;
         write->cinfo.err = jpeg_std_error( &write->eman.pub );
 	write->eman.pub.error_exit = new_error_exit;
@@ -381,7 +395,7 @@ write_exif( Write *write )
 	data_length = idl;
 
 #ifdef DEBUG
-	printf( "im_vips2jpeg: attaching %d bytes of EXIF\n", data_length  );
+	printf( "im_vips2jpeg: attaching %zd bytes of EXIF\n", data_length  );
 #endif /*DEBUG*/
 
 	exif_data_free( ed );
@@ -434,7 +448,7 @@ write_profile_data (j_compress_ptr cinfo,
   unsigned int length;          /* number of bytes to write in this marker */
 
   /* rounding up will fail for length == 0 */
-  assert( icc_data_len > 0 );
+  g_assert( icc_data_len > 0 );
 
   /* Calculate the number of markers we'll need, rounding up of course */
   num_markers = (icc_data_len + MAX_DATA_BYTES_IN_MARKER - 1) / 
@@ -487,7 +501,8 @@ static int
 write_profile_file( Write *write, const char *profile )
 {
 	if( !(write->profile_bytes = 
-		im__file_read_name( profile, &write->profile_length )) ) 
+		im__file_read_name( profile, VIPS_ICC_DIR, 
+		&write->profile_length )) ) 
 		return( -1 );
 	write_profile_data( &write->cinfo, 
 		(JOCTET *) write->profile_bytes, write->profile_length );
@@ -512,7 +527,7 @@ write_profile_meta( Write *write )
 	write_profile_data( &write->cinfo, data, data_length );
 
 #ifdef DEBUG
-	printf( "im_vips2jpeg: attached %d byte profile from VIPS header\n",
+	printf( "im_vips2jpeg: attached %zd byte profile from VIPS header\n",
 		data_length );
 #endif /*DEBUG*/
 
@@ -520,7 +535,7 @@ write_profile_meta( Write *write )
 }
 
 static int
-write_jpeg_block( REGION *region, Rect *area, void *a, void *b )
+write_jpeg_block( REGION *region, Rect *area, void *a )
 {
 	Write *write = (Write *) a;
 	int i;
@@ -554,9 +569,9 @@ write_vips( Write *write, int qfac, const char *profile )
 
 	/* Should have been converted for save.
 	 */
-        assert( in->BandFmt == IM_BANDFMT_UCHAR );
-	assert( in->Coding == IM_CODING_NONE );
-        assert( in->Bands == 1 || in->Bands == 3 || in->Bands == 4 );
+        g_assert( in->BandFmt == IM_BANDFMT_UCHAR );
+	g_assert( in->Coding == IM_CODING_NONE );
+        g_assert( in->Bands == 1 || in->Bands == 3 || in->Bands == 4 );
 
         /* Check input image.
          */
@@ -594,9 +609,8 @@ write_vips( Write *write, int qfac, const char *profile )
 
 	/* Build VIPS output stuff now we know the image we'll be writing.
 	 */
-	write->tg = im_threadgroup_create( in );
-	write->row_pointer = IM_ARRAY( NULL, write->tg->nlines, JSAMPROW );
-	if( !write->tg || !write->row_pointer ) 
+	if( !(write->row_pointer = 
+		IM_ARRAY( NULL, write->in->Ysize, JSAMPROW )) )
 		return( -1 );
 
 	/* Rest to default. 
@@ -627,7 +641,7 @@ write_vips( Write *write, int qfac, const char *profile )
 
 	/* Write data. Note that the write function grabs the longjmp()!
 	 */
-	if( im_wbuffer( write->tg, write_jpeg_block, write, NULL ) )
+	if( vips_sink_disc( write->in, write_jpeg_block, write ) )
 		return( -1 );
 
 	/* We have to reinstate the setjmp() before we jpeg_finish_compress().
@@ -640,7 +654,56 @@ write_vips( Write *write, int qfac, const char *profile )
 	return( 0 );
 }
 
-/* Write a VIPS image to a file as JPEG.
+/**
+ * im_vips2jpeg:
+ * @in: image to save 
+ * @filename: file to write to 
+ *
+ * Write a VIPS image to a file as JPEG.
+ *
+ * You can embed options in the filename. They have the form:
+ *
+ * |[
+ * filename.jpg:<emphasis>compression</emphasis>,<emphasis>profile</emphasis>
+ * ]|
+ *
+ * <itemizedlist>
+ *   <listitem>
+ *     <para>
+ * <emphasis>compression</emphasis> 
+ * Compress with this quality factor. Default 75.
+ *     </para>
+ *   </listitem>
+ *   <listitem>
+ *     <para>
+ * <emphasis>profile</emphasis> 
+ * Attach this ICC profile. For example, "fred.jpg:,/home/john/srgb.icc" will 
+ * embed the profile stored in the file "/home/john/srgb.icc" in the JPEG 
+ * image. This does not affect the pixels which are written, just the way 
+ * they are tagged. You can use the special string "none" to mean 
+ * "don't attach a profile".
+ *     </para>
+ *   </listitem>
+ * </itemizedlist>
+ *
+ * If no profile is specified in the save string and the VIPS header 
+ * contains an ICC profile named IM_META_ICC_NAME ("icc-profile-data"), the
+ * profile from the VIPS header will be attached.
+ *
+ * The image is automatically converted to RGB, Monochrome or CMYK before 
+ * saving. Any metadata attached to the image is saved as EXIF, if possible.
+ *
+ * Example:
+ *
+ * |[
+ * im_vips2jpeg( in, "fred.jpg:99,none" );
+ * ]|
+ *
+ * Will write "fred.jpg" at high-quality with no ICC profile.
+ *
+ * See also: #VipsFormat, im_jpeg2vips().
+ *
+ * Returns: 0 on success, -1 on error.
  */
 int
 im_vips2jpeg( IMAGE *in, const char *filename )
@@ -709,6 +772,112 @@ im_vips2jpeg( IMAGE *in, const char *filename )
 	return( 0 );
 }
 
+/* We can't predict how large the output buffer we need is, because we might
+ * need space for ICC profiles and stuff. So we write to a linked list of mem
+ * buffers and add a new one as they fill.
+ */
+
+#define BUFFER_SIZE (10000)
+
+/* A buffer.
+ */
+typedef struct _Block {
+	j_compress_ptr cinfo;
+
+	struct _Block *first;
+	struct _Block *next;
+
+	JOCTET *data;		/* Allocated area */
+	int size;		/* Max size */
+	int used;		/* How much has been used */
+} Block;
+
+static Block *
+block_new( j_compress_ptr cinfo )
+{
+	Block *block;
+
+	block = (Block *) (*cinfo->mem->alloc_large) 
+		( (j_common_ptr) cinfo, JPOOL_IMAGE, sizeof( Block ) );
+
+	block->cinfo = cinfo;
+	block->first = block;
+	block->next = NULL;
+	block->data = (JOCTET *) (*cinfo->mem->alloc_large) 
+		( (j_common_ptr) cinfo, JPOOL_IMAGE, BUFFER_SIZE );
+	block->size = BUFFER_SIZE;
+	block->used = 0;
+
+	return( block );
+}
+
+static Block *
+block_last( Block *block )
+{
+	while( block->next )
+		block = block->next;
+
+	return( block );
+}
+
+static Block *
+block_append( Block *block )
+{
+	Block *new;
+
+	g_assert( block );
+
+	new = block_new( block->cinfo );
+	new->first = block->first;
+	block_last( block )->next = new;
+
+	return( new );
+}
+
+static int
+block_length( Block *block )
+{
+	int len;
+
+	len = 0;
+	for( block = block->first; block; block = block->next )
+		len += block->used;
+
+	return( len );
+}
+
+static void
+block_copy( Block *block, void *dest )
+{
+	JOCTET *p;
+	
+	p = dest;
+	for( block = block->first; block; block = block->next ) {
+		memcpy( p, block->data, block->used );
+		p += block->used;
+	}
+}
+
+#ifdef DEBUG
+static void
+block_print( Block *block )
+{
+	int i;
+
+	printf( "total length = %d\n", block_length( block ) );
+	printf( "set of blocks:\n" );
+
+	i = 0;
+	for( block = block->first; block; block = block->next ) {
+		printf( "%d) %p, first = %p, next = %p"
+			"\t data = %p, size = %d, used = %d\n", 
+			i, block, block->first, block->next,
+			block->data, block->size, block->used );
+		i += 1;
+	}
+}
+#endif /*DEBUG*/
+
 /* Just like the above, but we write to a memory buffer.
  *
  * A memory buffer for the compressed image.
@@ -720,9 +889,10 @@ typedef struct {
 
 	/* Private stuff during write.
 	 */
-	JOCTET *data;		/* Allocated area */
-	int used;		/* Number of bytes written so far */
-	int size;		/* Max size */
+
+	/* Build the output area here in chunks.
+	 */
+	Block *block;
 
 	/* Copy the compressed area here.
 	 */
@@ -737,51 +907,72 @@ METHODDEF(void)
 init_destination( j_compress_ptr cinfo )
 {
 	OutputBuffer *buf = (OutputBuffer *) cinfo->dest;
-	int mx = cinfo->image_width * cinfo->image_height * 
-		cinfo->input_components * sizeof( JOCTET );
 
 	/* Allocate relative to the image we are writing .. freed when we junk
 	 * this output.
 	 */
-	buf->data = (JOCTET *) (*cinfo->mem->alloc_large) 
-			( (j_common_ptr) cinfo, JPOOL_IMAGE, mx );
-	buf->used = 0;
-	buf->size = mx;
+	buf->block = block_new( cinfo );
 
 	/* Set buf pointers for library.
 	 */
-	buf->pub.next_output_byte = buf->data;
-	buf->pub.free_in_buffer = mx;
+	buf->pub.next_output_byte = buf->block->data;
+	buf->pub.free_in_buffer = buf->block->size;
 }
 
-/* Buffer full method ... should never get this.
+/* Buffer full method ... allocate a new output block.
  */
 METHODDEF(boolean)
 empty_output_buffer( j_compress_ptr cinfo )
 {
-	/* Not really a file write error, but why not. Should never happen.
-	 */
-	ERREXIT( cinfo, JERR_FILE_WRITE );
+	OutputBuffer *buf = (OutputBuffer *) cinfo->dest;
 
-	return( 0 );
+	/* Record how many bytes we used. empty_output_buffer() is always
+	 * called when the buffer is exactly full.
+	 */
+	buf->block->used = buf->block->size;
+
+	/* New block and reset.
+	 */
+	buf->block = block_append( buf->block );
+	buf->pub.next_output_byte = buf->block->data;
+	buf->pub.free_in_buffer = buf->block->size;
+
+	/* TRUE means we've made some more space.
+	 */
+	return( 1 );
 }
 
-/* Cleanup. Write entire buffer as a MIME type.
+/* Cleanup. Copy the set of blocks out as a big lump.
  */
 METHODDEF(void)
 term_destination( j_compress_ptr cinfo )
 {
         OutputBuffer *buf = (OutputBuffer *) cinfo->dest;
-	int len = buf->size - buf->pub.free_in_buffer;
+
+	int len;
 	void *obuf;
 
-	/* Allocate and copy to the VIPS output area.
+	/* Record the number of bytes we wrote in the final buffer.
+	 * pub.free_in_buffer is valid here.
+	 */
+	buf->block->used = buf->block->size - buf->pub.free_in_buffer;
+
+#ifdef DEBUG
+	block_print( buf->block );
+#endif /*DEBUG*/
+
+	/* ... and we can count up our buffers now.
+	 */
+	len = block_length( buf->block );
+
+	/* Allocate and copy to the output area.
 	 */
 	if( !(obuf = im_malloc( buf->out, len )) )
 		ERREXIT( cinfo, JERR_FILE_WRITE );
-	memcpy( obuf, buf->data, len );
 	*(buf->obuf) = obuf;
 	*(buf->olen) = len;
+
+	block_copy( buf->block, obuf );
 }
 
 /* Set dest to one of our objects.
@@ -818,12 +1009,24 @@ buf_dest( j_compress_ptr cinfo, IMAGE *out, char **obuf, int *olen )
 	buf->olen = olen;
 }
 
-/* As above, but save to a buffer. The buffer is allocated relative to out.
- * On success, buf is set to the output buffer and len to the size of the
- * compressed image.
-
- 	FIXME ... argh, the retuen length should really be a size_t, but we can't fix this now sadly
-
+/**
+ * im_vips2bufjpeg:
+ * @in: image to save 
+ * @out: allocate output buffer local to this
+ * @qfac: JPEG quality factor
+ * @obuf: return output buffer here
+ * @olen: return output length here
+ *
+ * As im_vips2jpeg(), but save as a memory buffer. The memory is allocated
+ * local to @out (that is, when @out is closed the memory will be released,
+ * pass %NULL to release yourself). 
+ *
+ * The address of the buffer is returned in @obuf, the length of the buffer in
+ * @olen. @olen should really be a size_t rather than an int :-(
+ *
+ * See also: #VipsFormat, im_vips2jpeg().
+ *
+ * Returns: 0 on success, -1 on error.
  */
 int
 im_vips2bufjpeg( IMAGE *in, IMAGE *out, int qfac, char **obuf, int *olen )
@@ -865,7 +1068,16 @@ im_vips2bufjpeg( IMAGE *in, IMAGE *out, int qfac, char **obuf, int *olen )
 	return( 0 );
 }
 
-/* As above, but save as a mime jpeg on stdout.
+/**
+ * im_vips2mimejpeg:
+ * @in: image to save 
+ * @qfac: JPEG quality factor
+ *
+ * As im_vips2jpeg(), but save as a mime jpeg on stdout.
+ *
+ * See also: #VipsFormat, im_vips2jpeg().
+ *
+ * Returns: 0 on success, -1 on error.
  */
 int
 im_vips2mimejpeg( IMAGE *in, int qfac )
