@@ -9,6 +9,9 @@
  *	- fix signed/unsigned warnings
  * 12/10/09
  *	- heh argh reading history always stopped after the first line
+ * 9/12/09
+ * 	- only wholly map input files on im_incheck() ... this reduces VM use,
+ * 	  especially with large numbers of small files
  */
 
 /*
@@ -167,6 +170,7 @@
 
 /**
  * VipsBandFmt: 
+ * @IM_BANDFMT_NOTSET: invalid setting
  * @IM_BANDFMT_UCHAR: unsigned char format
  * @IM_BANDFMT_CHAR: char format
  * @IM_BANDFMT_USHORT: unsigned short format
@@ -332,15 +336,6 @@
  */
 #define NAMESPACE "http://www.vips.ecs.soton.ac.uk/vips" 
 
-/* mmap() whole vs. window threshold ... an int, so we can tune easily from a
- * debugger.
- */
-#ifdef DEBUG
-int im__mmap_limit = 1;
-#else
-int im__mmap_limit = IM__MMAP_LIMIT;
-#endif /*DEBUG*/
-
 /* Sort of open for read for image files. Shared with im_binfile().
  */
 int
@@ -824,7 +819,7 @@ rebuild_header( IMAGE *im )
 	return( 0 );
 }
 
-/* Called at the end of im__read_header ... get any XML after the pixel data
+/* Called at the end of im_openin() ... get any XML after the pixel data
  * and read it in.
  */
 static int 
@@ -1127,8 +1122,8 @@ im__writehist( IMAGE *im )
 
 /* Open the filename, read the header, some sanity checking.
  */
-static int
-im__read_header( IMAGE *image )
+int
+im_openin( IMAGE *image )
 {
 	/* We don't use im->sizeof_header here, but we know we're reading a
 	 * VIPS image anyway.
@@ -1143,7 +1138,7 @@ im__read_header( IMAGE *image )
 		return( -1 );
 	if( read( image->fd, header, IM_SIZEOF_HEADER ) != IM_SIZEOF_HEADER ||
 		im__read_header_bytes( image, header ) ) {
-		im_error( "im_openin", 
+		im_error( "im__read_header", 
 			_( "unable to read header for \"%s\", %s" ),
 			image->filename, strerror( errno ) );
 		return( -1 );
@@ -1156,7 +1151,8 @@ im__read_header( IMAGE *image )
 		return( -1 );
 	image->file_length = rsize;
 	if( psize > rsize ) 
-		im_warn( "im_openin", _( "unable to read data for \"%s\", %s" ),
+		im_warn( "im__read_header", 
+			_( "unable to read data for \"%s\", %s" ),
 			image->filename, _( "file has been truncated" ) );
 
 	/* Set demand style. Allow the most permissive sort.
@@ -1168,7 +1164,7 @@ im__read_header( IMAGE *image )
 	 * harmless.
 	 */
 	if( im__readhist( image ) ) {
-		im_warn( "im_openin", _( "error reading XML: %s" ),
+		im_warn( "im__read_header", _( "error reading XML: %s" ),
 			im_error_buffer() );
 		im_error_clear();
 	}
@@ -1176,61 +1172,13 @@ im__read_header( IMAGE *image )
 	return( 0 );
 }
 
-/* Open, then mmap() small images, leave large images to have a rolling mmap()
- * window for each region. This is old and deprecated API, use im_vips_open()
- * in preference.
- */
-int
-im_openin( IMAGE *image )
-{
-	gint64 size;
-
-#ifdef DEBUG
-	char *str;
-
-	if( (str = g_getenv( "IM_MMAP_LIMIT" )) ) {
-		im__mmap_limit = atoi( str );
-		printf( "im_openin: setting maplimit to %d from environment\n",
-			im__mmap_limit );
-	}
-#endif /*DEBUG*/
-
-	if( im__read_header( image ) )
-		return( -1 );
-
-	/* Make sure we can map the whole thing without running over the VM
-	 * limit or running out of file.
-	 */
-	size = (gint64) IM_IMAGE_SIZEOF_LINE( image ) * image->Ysize + 
-		image->sizeof_header;
-	if( size < im__mmap_limit && image->file_length >= size ) {
-		if( im_mapfile( image ) )
-			return( -1 );
-		image->data = image->baseaddr + image->sizeof_header;
-		image->dtype = IM_MMAPIN;
-
-#ifdef DEBUG
-		printf( "im_openin: completely mmap()ing \"%s\": it's small\n",
-			image->filename );
-#endif /*DEBUG*/
-	}
-	else {
-#ifdef DEBUG
-		printf( "im_openin: delaying mmap() of \"%s\": it's big!\n",
-			image->filename );
-#endif /*DEBUG*/
-	}
-
-	return( 0 );
-}
-
 /* Open, then mmap() read/write. This is old and deprecated API, use
  * im_vips_open() in preference.
  */
 int
 im_openinrw( IMAGE *image )
 {
-	if( im__read_header( image ) )
+	if( im_openin( image ) )
 		return( -1 );
 	if( im_mapfilerw( image ) ) 
 		return( -1 );
@@ -1238,7 +1186,7 @@ im_openinrw( IMAGE *image )
 	image->dtype = IM_MMAPINRW;
 
 #ifdef DEBUG
-	printf( "im_openin: completely mmap()ing \"%s\" read-write\n",
+	printf( "im_openinrw: completely mmap()ing \"%s\" read-write\n",
 		image->filename );
 #endif /*DEBUG*/
 
@@ -1279,47 +1227,29 @@ im_open_vips( const char *filename )
 		}
 	}
 
-	/* Not in native format?
+	/* Not in native format? And needs swapping?
 	 */
-	if( im_isMSBfirst( im ) != im_amiMSBfirst() ) {
-		/* Does it need swapping? 
-		 */
-		switch( im->Coding ) {
-		case IM_CODING_LABQ:
-		case IM_CODING_RAD:
-			break;
+	if( im_isMSBfirst( im ) != im_amiMSBfirst() &&
+		im->Coding == IM_CODING_NONE &&
+		im->BandFmt != IM_BANDFMT_CHAR &&
+		im->BandFmt != IM_BANDFMT_UCHAR ) {
+		IMAGE *im2;
 
-		case IM_CODING_NONE:
-			if( im->BandFmt != IM_BANDFMT_CHAR &&
-				im->BandFmt != IM_BANDFMT_UCHAR ) {
-				IMAGE *im2;
-
-				/* Needs swapping :( make a little pipeline up 
-				 * to do this for us.
-				 */
-				if( !(im2 = im_open( filename, "p" )) ) {
-					im_close( im );
-					return( NULL );
-				}
-				if( im_add_close_callback( im2, 
-					(im_callback_fn)im_close, im, NULL ) ) {
-					im_close( im );
-					im_close( im2 );
-					return( NULL );
-				}
-				if( im_copy_swap( im, im2 ) ) {
-					im_close( im2 );
-					return( NULL );
-				}
-				im = im2;
-			}
-			break;
-
-		default:
+		if( !(im2 = im_open( filename, "p" )) ) {
 			im_close( im );
-			im_error( "im_open", "%s", _( "unknown coding type" ) );
 			return( NULL );
 		}
+		if( im_add_close_callback( im2, 
+			(im_callback_fn) im_close, im, NULL ) ) {
+			im_close( im );
+			im_close( im2 );
+			return( NULL );
+		}
+		if( im_copy_swap( im, im2 ) ) {
+			im_close( im2 );
+			return( NULL );
+		}
+		im = im2;
 	}
 
 	return( im );
