@@ -32,6 +32,10 @@
  *	- add file_length 
  * 8/10/09
  * 	- add set_hint
+ * 10/1/09
+ * 	- added postclose
+ * 14/1/09
+ * 	- added writtenfns
  */
 
 /*
@@ -74,8 +78,8 @@
 #include <string.h>
 
 #include <vips/vips.h>
-#include <vips/debug.h>
 #include <vips/thread.h>
+#include <vips/debug.h>
 #include <vips/internal.h>
 
 #ifdef WITH_DMALLOC
@@ -157,8 +161,8 @@ im_init( const char *filename )
 
 	im->windows = NULL;
 
-	im->parents = NULL;
-	im->children = NULL;
+	im->upstream = NULL;
+	im->downstream = NULL;
 	im->serial = 0;
 
 	im->history_list = NULL;
@@ -173,6 +177,9 @@ im_init( const char *filename )
 
 	im->hint_set = FALSE;
 
+	im->postclosefns = NULL;
+	im->writtenfns = NULL;
+
 	if( !(im->filename = im_strdup( NULL, filename )) ) {
 		im_close( im );
 		return( NULL );
