@@ -817,7 +817,7 @@ im__write( int fd, const void *buf, size_t count )
 /* Load up a file as a string.
  */
 char *
-im__file_read( FILE *fp, const char *name, unsigned int *length_out )
+im__file_read( FILE *fp, const char *filename, unsigned int *length_out )
 {
         long len;
 	size_t read;
@@ -830,7 +830,7 @@ im__file_read( FILE *fp, const char *name, unsigned int *length_out )
 	if( len > 20 * 1024 * 1024 ) {
 		/* Seems crazy!
 		 */
-                im_error( "im__file_read", _( "\"%s\" too long" ), name );
+                im_error( "im__file_read", _( "\"%s\" too long" ), filename );
                 return( NULL );
         }
 
@@ -860,7 +860,7 @@ im__file_read( FILE *fp, const char *name, unsigned int *length_out )
 		} while( !feof( fp ) );
 
 #ifdef DEBUG
-		printf( "read %d bytes from unseekable stream\n", len );
+		printf( "read %ld bytes from unseekable stream\n", len );
 #endif /*DEBUG*/
 	}
 	else {
@@ -873,7 +873,8 @@ im__file_read( FILE *fp, const char *name, unsigned int *length_out )
 		if( read != (size_t) len ) {
 			im_free( str );
 			im_error( "im__file_read", 
-				_( "error reading from file \"%s\"" ), name );
+				_( "error reading from file \"%s\"" ), 
+				filename );
 			return( NULL );
 		}
 	}
@@ -886,23 +887,60 @@ im__file_read( FILE *fp, const char *name, unsigned int *length_out )
         return( str );
 }
 
+/* Does a filename contain a directory separator?
+ */
+static gboolean 
+filename_hasdir( const char *filename )
+{
+	char *dirname;
+	gboolean hasdir;
+
+	dirname = g_path_get_dirname( filename );
+	hasdir = (strcmp( dirname, "." ) != 0);
+	g_free( dirname );
+
+	return( hasdir );
+}
+
+/* Open a file. We take an optional fallback dir as well and will try opening
+ * there if opening directly fails.
+ *
+ * This is used for things like finding ICC profiles. We try to open the file
+ * directly first, and if that fails and the filename does not contain a
+ * directory separator, we try looking in the fallback dir.
+ */
 FILE *
-im__file_open_read( const char *filename )
+im__file_open_read( const char *filename, const char *fallback_dir )
 {
 	FILE *fp;
 
 #ifdef BINARY_OPEN
-        if( !(fp = fopen( filename, "rb" )) ) {
-#else /*BINARY_OPEN*/
-        if( !(fp = fopen( filename, "r" )) ) {
+        fp = fopen( filename, "rb" );
+#else /*!BINARY_OPEN*/
+        fp = fopen( filename, "r" );
 #endif /*BINARY_OPEN*/
-		im_error( "im__file_open_read", 
-			_( "unable to open file \"%s\" for reading" ), 
-			filename );
-		return( NULL );
+	if( fp )
+		return( fp );
+
+	if( fallback_dir && !filename_hasdir( filename ) ) {
+		char *path;
+
+		path = g_build_filename( fallback_dir, filename, NULL );
+#ifdef BINARY_OPEN
+	        fp = fopen( path, "rb" );
+#else /*!BINARY_OPEN*/
+        	fp = fopen( path, "r" );
+#endif /*BINARY_OPEN*/
+		g_free( path );
+
+		if( fp )
+			return( fp );
 	}
 
-	return( fp );
+	im_error( "im__file_open_read", 
+		_( "unable to open file \"%s\" for reading" ), filename );
+
+	return( NULL );
 }
 
 FILE *
@@ -924,17 +962,19 @@ im__file_open_write( const char *filename )
 	return( fp );
 }
 
-/* Load from a filename as a string.
+/* Load from a filename as a string. Used for things like reading in ICC
+ * profiles.
  */
 char *
-im__file_read_name( const char *name, unsigned int *length_out )
+im__file_read_name( const char *filename, const char *fallback_dir, 
+	unsigned int *length_out )
 {
 	FILE *fp;
 	char *buffer;
 
-        if( !(fp = im__file_open_read( name )) ) 
+        if( !(fp = im__file_open_read( filename, fallback_dir )) ) 
 		return( NULL );
-	if( !(buffer = im__file_read( fp, name, length_out )) ) {
+	if( !(buffer = im__file_read( fp, filename, length_out )) ) {
 		fclose( fp );
 		return( NULL );
 	}
@@ -1186,26 +1226,6 @@ im__file_write( void *data, size_t size, size_t nmemb, FILE *stream )
 	return( 0 );
 }
 
-/* Check whether arch corresponds to native byte order.
- */
-gboolean
-im_isnative( im_arch_type arch )
-{
-	switch( arch ) {
-	case IM_ARCH_NATIVE: 		
-		return( TRUE );
-	case IM_ARCH_BYTE_SWAPPED: 	
-		return( FALSE );
-	case IM_ARCH_LSB_FIRST: 	
-		return( !im_amiMSBfirst() );
-	case IM_ARCH_MSB_FIRST: 	
-		return( im_amiMSBfirst() );
-
-	default:
-		g_assert( 0 );
-	}  
-}
-
 /* Read a few bytes from the start of a file. For sniffing file types.
  */
 int
@@ -1413,6 +1433,36 @@ im_existsf( const char *name, ... )
         return( 0 );
 }
 
+#ifdef OS_WIN32
+#define popen(b,m) _popen(b,m)
+#define pclose(f) _pclose(f)
+#endif /*OS_WIN32*/
+
+/* Do popen(), with printf-style args.
+ */
+FILE *
+im_popenf( const char *fmt, const char *mode, ... )
+{
+        va_list args;
+	char buf[4096];
+	FILE *fp;
+
+        va_start( args, mode );
+        (void) im_vsnprintf( buf, 4096, fmt, args );
+        va_end( args );
+
+#ifdef DEBUG
+	printf( "im_popenf: running: %s\n", buf );
+#endif /*DEBUG*/
+
+        if( !(fp = popen( buf, mode )) ) {
+		im_error( "popenf", "%s", strerror( errno ) );
+		return( NULL );
+	}
+
+	return( fp );
+}
+
 /* True if an int is a power of two ... 1, 2, 4, 8, 16, 32, etc. Do with just
  * integer arithmetic for portability. A previous Nicos version using doubles
  * and log/log failed on x86 with rounding problems. Return 0 for not
@@ -1478,39 +1528,90 @@ im_amiMSBfirst( void )
                 return( 1 );
 }
 
-/* Make a disc IMAGE which will be automatically unlinked on im_close().
+/* Return the tmp dir. On Windows, GetTempPath() will also check the values of 
+ * TMP, TEMP and USERPROFILE.
  */
-IMAGE *
-im__open_temp( void )
+static const char *
+im__temp_dir( void )
 {
 	const char *tmpd;
+
+	if( !(tmpd = g_getenv( "TMPDIR" )) ) {
+#ifdef OS_WIN32
+		static gboolean done = FALSE;
+		static char buf[256];
+
+		if( !done ) {
+			if( !GetTempPath( 256, buf ) )
+				strcpy( buf, "C:\\temp" );
+		}
+		tmpd = buf;
+#else /*!OS_WIN32*/
+		tmpd = "/tmp";
+#endif /*!OS_WIN32*/
+	}
+
+	return( tmpd );
+}
+
+/* Make a temporary file name. The format parameter is something like "%s.jpg" 
+ * and will be expanded to something like "/tmp/vips-12-34587.jpg".
+ *
+ * You need to free the result. A real file will also be created, though we
+ * delete it for you.
+ */
+char *
+im__temp_name( const char *format )
+{
+	static int serial = 1;
+
+	char file[FILENAME_MAX];
+	char file2[FILENAME_MAX];
+
 	char *name;
 	int fd;
-	IMAGE *disc;
 
-	if( !(tmpd = g_getenv( "TMPDIR" )) )
-		tmpd = "/tmp";
-	name = g_build_filename( tmpd, "vips_XXXXXX.v", NULL );
+	im_snprintf( file, FILENAME_MAX, "vips-%d-XXXXXX", serial++ );
+	im_snprintf( file2, FILENAME_MAX, format, file );
+	name = g_build_filename( im__temp_dir(), file2, NULL );
 
 	if( (fd = g_mkstemp( name )) == -1 ) {
 		im_error( "tempfile", 
-			_( "unable to make temp file %s" ), name );
+			_( "unable to make temporary file %s" ), name );
 		g_free( name );
 		return( NULL );
 	}
 	close( fd );
+	g_unlink( name );
+
+	return( name );
+}
+
+/* Make a disc IMAGE which will be automatically unlinked on im_close().
+ * Format is something like "%s.v" for a vips file.
+ */
+IMAGE *
+im__open_temp( const char *format )
+{
+	char *name;
+	IMAGE *disc;
+
+	if( !(name = im__temp_name( format )) )
+		return( NULL );
 
 	if( !(disc = im_open( name, "w" )) ) {
-		unlink( name );
 		g_free( name );
 		return( NULL );
 	}
 	g_free( name );
 
-	if( im_add_close_callback( disc, 
+	/* Needs to be postclose so we can rewind after write without
+	 * deleting the file.
+	 */
+	if( im_add_postclose_callback( disc, 
 		(im_callback_fn) unlink, disc->filename, NULL ) ) {
 		im_close( disc );
-		unlink( name );
+		g_unlink( name );
 	}
 
 	return( disc );
