@@ -129,7 +129,7 @@ find_top( REGION *ir, int *pos, int x, int y, int h )
 
 	/* Double the number of bands in a complex.
 	 */
-	if( im_iscomplex( im ) )
+	if( vips_bandfmt_iscomplex( im->BandFmt ) )
 		b *= 2;
 
 /* Search for the first non-zero band element from the top edge of the image.
@@ -183,7 +183,7 @@ find_bot( REGION *ir, int *pos, int x, int y, int h )
 
 	/* Double the number of bands in a complex.
 	 */
-	if( im_iscomplex( im ) )
+	if( vips_bandfmt_iscomplex( im->BandFmt ) )
 		b *= 2;
 
 /* Search for the first non-zero band element from the top edge of the image.
