@@ -1,12 +1,18 @@
 /* vipsinterpolate ... abstract base class for various interpolators
  *
  * J. Cupitt, 15/10/08
+ *
+ * 12/8/10
+ * 	- revise window_size / window_offset stuff again: window_offset now
+ * 	  defaults to (window_size / 2 - 1), so for a 4x4 stencil (eg.
+ * 	  bicubic) we have an offset of 1
+ * 	- tiny speedups
  */
 
 /*
 
     This file is part of VIPS.
-    
+
     VIPS is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
@@ -48,7 +54,7 @@
 #include <dmalloc.h>
 #endif /*WITH_DMALLOC*/
 
-/** 
+/**
  * SECTION: interpolate
  * @short_description: various interpolators: nearest, bilinear, bicubic, and
  * some non-linear
@@ -107,9 +113,21 @@ vips_interpolate_real_get_window_size( VipsInterpolate *interpolate )
 static int
 vips_interpolate_real_get_window_offset( VipsInterpolate *interpolate )
 {
-	/* Default to half window size.
-	 */
-	return( vips_interpolate_get_window_size( interpolate ) / 2 );
+	VipsInterpolateClass *class = VIPS_INTERPOLATE_GET_CLASS( interpolate );
+
+ 	/* Default to half window size - 1. For example, bicubic is a 4x4
+	 * stencil and needs an offset of 1.
+ 	 */
+	if( class->window_offset != -1 )
+		return( class->window_offset );
+	else {
+		int window_size = 
+			vips_interpolate_get_window_size( interpolate );
+
+		/* Don't go -ve, of course, for window_size 1.
+		 */
+		return( IM_MAX( 0, window_size / 2 - 1 ) );
+	}
 }
 
 static void
@@ -126,6 +144,7 @@ vips_interpolate_class_init( VipsInterpolateClass *class )
 	class->get_window_size = vips_interpolate_real_get_window_size;
 	class->get_window_offset = vips_interpolate_real_get_window_offset;
 	class->window_size = -1;
+	class->window_offset = -1;
 }
 
 static void
@@ -141,19 +160,20 @@ vips_interpolate_init( VipsInterpolate *interpolate )
  * in_x, in_y in REGION in. Don't do this as a signal for speed.
  */
 void
-vips_interpolate( VipsInterpolate *interpolate, 
+vips_interpolate( VipsInterpolate *interpolate,
 	PEL *out, REGION *in, double x, double y )
 {
 	VipsInterpolateClass *class = VIPS_INTERPOLATE_GET_CLASS( interpolate );
 
 	g_assert( class->interpolate );
+
 	class->interpolate( interpolate, out, in, x, y );
 }
 
 /* As above, but return the function pointer. Use this to cache method
  * dispatch.
  */
-VipsInterpolateMethod 
+VipsInterpolateMethod
 vips_interpolate_get_method( VipsInterpolate *interpolate )
 {
 	VipsInterpolateClass *class = VIPS_INTERPOLATE_GET_CLASS( interpolate );
@@ -210,24 +230,26 @@ vips_interpolate_get_window_offset( VipsInterpolate *interpolate )
 typedef VipsInterpolate VipsInterpolateNearest;
 typedef VipsInterpolateClass VipsInterpolateNearestClass;
 
-G_DEFINE_TYPE( VipsInterpolateNearest, vips_interpolate_nearest, 
+G_DEFINE_TYPE( VipsInterpolateNearest, vips_interpolate_nearest,
 	VIPS_TYPE_INTERPOLATE );
 
 static void
-vips_interpolate_nearest_interpolate( VipsInterpolate *interpolate, 
+vips_interpolate_nearest_interpolate( VipsInterpolate *interpolate,
 	PEL *out, REGION *in, double x, double y )
 {
 	/* Pel size and line size.
 	 */
 	const int ps = IM_IMAGE_SIZEOF_PEL( in->im );
-	int z;
 
-	/* Top left corner we interpolate from.
+	/* Top left corner we interpolate from. We know x/y are always
+	 * positive, so we can just (int) them.
 	 */
-	const int xi = FAST_PSEUDO_FLOOR( x );
-	const int yi = FAST_PSEUDO_FLOOR( y );
+	const int xi = (int) x;
+	const int yi = (int) y;
 
-	const PEL *p = (PEL *) IM_REGION_ADDR( in, xi, yi ); 
+	const PEL *p = (PEL *) IM_REGION_ADDR( in, xi, yi );
+
+	int z;
 
 	for( z = 0; z < ps; z++ )
 		out[z] = p[z];
@@ -237,7 +259,7 @@ static void
 vips_interpolate_nearest_class_init( VipsInterpolateNearestClass *class )
 {
 	VipsObjectClass *object_class = VIPS_OBJECT_CLASS( class );
-	VipsInterpolateClass *interpolate_class = 
+	VipsInterpolateClass *interpolate_class =
 		VIPS_INTERPOLATE_CLASS( class );
 
 	object_class->nickname = "nearest";
@@ -260,7 +282,7 @@ VipsInterpolate *
 vips_interpolate_nearest_new( void )
 {
 
-	return( VIPS_INTERPOLATE( vips_object_new( 
+	return( VIPS_INTERPOLATE( vips_object_new(
 		VIPS_TYPE_INTERPOLATE_NEAREST, NULL, NULL, NULL ) ) );
 }
 
@@ -296,36 +318,36 @@ vips_interpolate_nearest_static( void )
 	VIPS_TYPE_INTERPOLATE_BILINEAR, VipsInterpolateBilinearClass ))
 
 typedef VipsInterpolate VipsInterpolateBilinear;
+typedef VipsInterpolateClass VipsInterpolateBilinearClass;
 
-typedef struct _VipsInterpolateBilinearClass {
-	VipsInterpolateClass parent_class;
-
-	/* Precalculated interpolation matricies. int (used for pel sizes up 
-	 * to short), and float (for all others). We go to scale + 1, so
-	 * we can round-to-nearest safely. Don't bother with double, since
-	 * this is an approximation anyway.
- 	 */
-	int matrixi[VIPS_TRANSFORM_SCALE + 1][VIPS_TRANSFORM_SCALE + 1][4];
-	float matrixd[VIPS_TRANSFORM_SCALE + 1][VIPS_TRANSFORM_SCALE + 1][4];
-} VipsInterpolateBilinearClass;
-
-G_DEFINE_TYPE( VipsInterpolateBilinear, vips_interpolate_bilinear, 
+G_DEFINE_TYPE( VipsInterpolateBilinear, vips_interpolate_bilinear,
 	VIPS_TYPE_INTERPOLATE );
 
+/* Precalculated interpolation matricies. int (used for pel sizes up
+ * to short), and float (for all others). We go to scale + 1 so
+ * we can round-to-nearest safely. Don't bother with double, since
+ * this is an approximation anyway.
+ */
+static int vips_bilinear_matrixi
+	[VIPS_TRANSFORM_SCALE + 1][VIPS_TRANSFORM_SCALE + 1][4];
+static float vips_bilinear_matrixd
+	[VIPS_TRANSFORM_SCALE + 1][VIPS_TRANSFORM_SCALE + 1][4];
+
 /* in this class, name vars in the 2x2 grid as eg.
  * p1  p2
  * p3  p4
- */ 
+ */
 
-/* Interpolate a section ... int8/16 types.
+/* Interpolate a section ... int8/16 types, lookup tables for interpolation 
+ * factors, fixed-point arithmetic.
  */
 #define BILINEAR_INT( TYPE ) { \
 	TYPE *tq = (TYPE *) out; \
  	\
-	const int c1 = class->matrixi[xi][yi][0]; \
-	const int c2 = class->matrixi[xi][yi][1]; \
-	const int c3 = class->matrixi[xi][yi][2]; \
-	const int c4 = class->matrixi[xi][yi][3]; \
+	const int c1 = vips_bilinear_matrixi[tx][ty][0]; \
+	const int c2 = vips_bilinear_matrixi[tx][ty][1]; \
+	const int c3 = vips_bilinear_matrixi[tx][ty][2]; \
+	const int c4 = vips_bilinear_matrixi[tx][ty][3]; \
  	\
 	const TYPE *tp1 = (TYPE *) p1; \
 	const TYPE *tp2 = (TYPE *) p2; \
@@ -337,15 +359,16 @@ G_DEFINE_TYPE( VipsInterpolateBilinear, vips_interpolate_bilinear,
 			 c3 * tp3[z] + c4 * tp4[z]) >> VIPS_INTERPOLATE_SHIFT; \
 }
 
-/* Interpolate a pel ... int32 and float types.
+/* Interpolate a pel ... int32 and float types, lookup tables, float 
+ * arithmetic.
  */
 #define BILINEAR_FLOAT( TYPE ) { \
 	TYPE *tq = (TYPE *) out; \
  	\
-	const double c1 = class->matrixd[xi][yi][0]; \
-	const double c2 = class->matrixd[xi][yi][1]; \
-	const double c3 = class->matrixd[xi][yi][2]; \
-	const double c4 = class->matrixd[xi][yi][3]; \
+	const double c1 = vips_bilinear_matrixd[tx][ty][0]; \
+	const double c2 = vips_bilinear_matrixd[tx][ty][1]; \
+	const double c3 = vips_bilinear_matrixd[tx][ty][2]; \
+	const double c4 = vips_bilinear_matrixd[tx][ty][3]; \
 	\
 	const TYPE *tp1 = (TYPE *) p1; \
 	const TYPE *tp2 = (TYPE *) p2; \
@@ -376,41 +399,43 @@ G_DEFINE_TYPE( VipsInterpolateBilinear, vips_interpolate_bilinear,
 }
 
 static void
-vips_interpolate_bilinear_interpolate( VipsInterpolate *interpolate, 
+vips_interpolate_bilinear_interpolate( VipsInterpolate *interpolate,
 	PEL *out, REGION *in, double x, double y )
 {
-	VipsInterpolateBilinearClass *class = 
-		VIPS_INTERPOLATE_BILINEAR_GET_CLASS( interpolate );
-
 	/* Pel size and line size.
 	 */
 	const int ps = IM_IMAGE_SIZEOF_PEL( in->im );
-	const int ls = IM_REGION_LSKIP( in ); 
-	const int b = in->im->Bands; 
+	const int ls = IM_REGION_LSKIP( in );
+	const int b = in->im->Bands;
 
-	/* Now go to scaled int. 
+	/* Find the mask index. We round-to-nearest, so we need to generate 
+	 * indexes in 0 to VIPS_TRANSFORM_SCALE, 2^n + 1 values. We multiply 
+	 * by 2 more than we need to, add one, mask, then shift down again to 
+	 * get the extra range.
 	 */
-	const double sx = x * VIPS_TRANSFORM_SCALE;
-	const double sy = y * VIPS_TRANSFORM_SCALE;
-	const int sxi = FAST_PSEUDO_FLOOR( sx );
-	const int syi = FAST_PSEUDO_FLOOR( sy );
+	const int sx = x * VIPS_TRANSFORM_SCALE * 2;
+	const int sy = y * VIPS_TRANSFORM_SCALE * 2;
 
-	/* Get index into interpolation table and unscaled integer 
-	 * position.
+	const int six = sx & (VIPS_TRANSFORM_SCALE * 2 - 1);
+	const int siy = sy & (VIPS_TRANSFORM_SCALE * 2 - 1);
+
+	const int tx = (six + 1) >> 1;
+	const int ty = (siy + 1) >> 1;
+
+	/* We want ((int)x) ... void repeating this double -> int conversion 
+	 * by just shifting sx down.
 	 */
-	const int xi = sxi & (VIPS_TRANSFORM_SCALE - 1);
-	const int yi = syi & (VIPS_TRANSFORM_SCALE - 1);
-	const int x_int = sxi >> VIPS_TRANSFORM_SHIFT;
-	const int y_int = syi >> VIPS_TRANSFORM_SHIFT;
+	const int ix = sx >> (VIPS_TRANSFORM_SHIFT + 1);
+	const int iy = sy >> (VIPS_TRANSFORM_SHIFT + 1);
 
-	const PEL *p1 = (PEL *) IM_REGION_ADDR( in, x_int, y_int ); 
+	const PEL *p1 = (PEL *) IM_REGION_ADDR( in, ix, iy );
 	const PEL *p2 = p1 + ps;
-	const PEL *p3 = p1 + ls; 
-	const PEL *p4 = p3 + ps; 
+	const PEL *p3 = p1 + ls;
+	const PEL *p4 = p3 + ps;
 
 	int z;
 
-	SWITCH_INTERPOLATE( in->im->BandFmt, 
+	SWITCH_INTERPOLATE( in->im->BandFmt,
 		BILINEAR_INT, BILINEAR_FLOAT );
 }
 
@@ -418,7 +443,7 @@ static void
 vips_interpolate_bilinear_class_init( VipsInterpolateBilinearClass *class )
 {
 	VipsObjectClass *object_class = VIPS_OBJECT_CLASS( class );
-	VipsInterpolateClass *interpolate_class = 
+	VipsInterpolateClass *interpolate_class =
 		(VipsInterpolateClass *) class;
 	int x, y;
 
@@ -439,7 +464,7 @@ vips_interpolate_bilinear_class_init( VipsInterpolateBilinearClass *class )
 			 */
 			X = (double) x / VIPS_TRANSFORM_SCALE;
 			Y = (double) y / VIPS_TRANSFORM_SCALE;
-			Xd = 1.0 - X;	
+			Xd = 1.0 - X;
 			Yd = 1.0 - Y;
 
 			/* Weights.
@@ -449,15 +474,19 @@ vips_interpolate_bilinear_class_init( VipsInterpolateBilinearClass *class )
 			c3 = Xd * Y;
 			c4 = X * Y;
 
-			class->matrixd[x][y][0] = c1;
-			class->matrixd[x][y][1] = c2;
-			class->matrixd[x][y][2] = c3;
-			class->matrixd[x][y][3] = c4;
+			vips_bilinear_matrixd[x][y][0] = c1;
+			vips_bilinear_matrixd[x][y][1] = c2;
+			vips_bilinear_matrixd[x][y][2] = c3;
+			vips_bilinear_matrixd[x][y][3] = c4;
 
-			class->matrixi[x][y][0] = c1 * VIPS_INTERPOLATE_SCALE;
-			class->matrixi[x][y][1] = c2 * VIPS_INTERPOLATE_SCALE;
-			class->matrixi[x][y][2] = c3 * VIPS_INTERPOLATE_SCALE;
-			class->matrixi[x][y][3] = c4 * VIPS_INTERPOLATE_SCALE;
+			vips_bilinear_matrixi[x][y][0] = 
+				c1 * VIPS_INTERPOLATE_SCALE;
+			vips_bilinear_matrixi[x][y][1] = 
+				c2 * VIPS_INTERPOLATE_SCALE;
+			vips_bilinear_matrixi[x][y][2] = 
+				c3 * VIPS_INTERPOLATE_SCALE;
+			vips_bilinear_matrixi[x][y][3] = 
+				c4 * VIPS_INTERPOLATE_SCALE;
 		}
 }
 
@@ -474,7 +503,7 @@ vips_interpolate_bilinear_init( VipsInterpolateBilinear *bilinear )
 VipsInterpolate *
 vips_interpolate_bilinear_new( void )
 {
-	return( VIPS_INTERPOLATE( vips_object_new( 
+	return( VIPS_INTERPOLATE( vips_object_new(
 		VIPS_TYPE_INTERPOLATE_BILINEAR, NULL, NULL, NULL ) ) );
 }
 
@@ -497,18 +526,19 @@ void
 vips__interpolate_init( void )
 {
 	extern GType vips_interpolate_bicubic_get_type( void );
-	extern GType vips_interpolate_yafrsmooth_get_type( void );
-	extern GType vips_interpolate_nohalo1_get_type( void );
-	extern GType vips_interpolate_snohalo1_get_type( void );
-	extern GType vips_interpolate_nohalo2_get_type( void );
+	extern GType vips_interpolate_lbb_get_type( void );
+	extern GType vips_interpolate_nohalo_get_type( void );
+	extern GType vips_interpolate_vsqbs_get_type( void );
 
 	vips_interpolate_nearest_get_type();
 	vips_interpolate_bilinear_get_type();
+
+#ifdef ENABLE_CXX
 	vips_interpolate_bicubic_get_type();
-	vips_interpolate_yafrsmooth_get_type();
-	vips_interpolate_nohalo1_get_type();
-	vips_interpolate_snohalo1_get_type();
-	vips_interpolate_nohalo2_get_type();
+	vips_interpolate_lbb_get_type();
+	vips_interpolate_nohalo_get_type();
+	vips_interpolate_vsqbs_get_type();
+#endif /*ENABLE_CXX*/
 }
 
 /* Make an interpolator from a nickname.
