@@ -32,6 +32,9 @@
  * 	- and now we just have --list packages/classes/package-name
  * 13/11/09
  * 	- drop _f postfixes, drop many postfixes
+ * 24/6/10
+ * 	- less chatty error messages
+ * 	- oops, don't rename "copy_set" as "copy_"
  */
 
 /*
@@ -92,21 +95,21 @@ static GOptionEntry main_option[] = {
 	{ "list", 'l', 0, G_OPTION_ARG_STRING, &main_option_list, 
 		N_( "list operations in PACKAGE "
 			"(or \"all\", \"packages\", \"classes\")" ),
-		"PACKAGE" },
+		N_( "PACKAGE" ) },
 	{ "usage", 'u', 0, G_OPTION_ARG_STRING, &main_option_usage, 
 		N_( "show usage message for OPERATION" ), 
-		"OPERATION" },
+		N_( "OPERATION" ) },
 	{ "plugin", 'p', 0, G_OPTION_ARG_FILENAME, &main_option_plugin, 
 		N_( "load PLUGIN" ), 
-		"PLUGIN" },
+		N_( "PLUGIN" ) },
 	{ "links", 'k', 0, G_OPTION_ARG_NONE, &main_option_links, 
 		N_( "print link lines for all operations" ), NULL },
 	{ "cpph", 'h', 0, G_OPTION_ARG_STRING, &main_option_cpph, 
 		N_( "print C++ decls for PACKAGE (or \"all\")" ), 
-		"PACKAGE" },
+		N_( "PACKAGE" ) },
 	{ "cppc", 'c', 0, G_OPTION_ARG_STRING, &main_option_cppc, 
 		N_( "print C++ binding for PACKAGE (or \"all\")" ), 
-		"PACKAGE" },
+		N_( "PACKAGE" ) },
 	{ "version", 'v', 0, G_OPTION_ARG_NONE, &main_option_version, 
 		N_( "print im_version_string" ), NULL },
 	{ NULL }
@@ -461,7 +464,10 @@ drop_postfix( char *str, const char *postfix )
 static void
 c2cpp_name( const char *in, char *out )
 {
-	static const char *postfix[] = {
+	static const char *dont_drop[] = {
+		"_set",
+	};
+	static const char *drop[] = {
 		"_vec",
 		"const",
 		"tra",
@@ -479,12 +485,24 @@ c2cpp_name( const char *in, char *out )
 	else
 		strcpy( out, in );
 
-	/* Repeatedly drop postfixes while we can.
+	/* Repeatedly drop postfixes while we can. Stop if we see a dont_drop
+	 * postfix.
 	 */
 	do {
+		gboolean found;
+
+		found = FALSE;
+		for( i = 0; i < IM_NUMBER( dont_drop ); i++ )
+			if( ispostfix( dont_drop[i], out ) ) {
+				found = TRUE;
+				break;
+			}
+		if( found )
+			break;
+
 		changed = FALSE;
-		for( i = 0; i < IM_NUMBER( postfix ); i++ )
-			changed |= drop_postfix( out, postfix[i] );
+		for( i = 0; i < IM_NUMBER( drop ); i++ )
+			changed |= drop_postfix( out, drop[i] );
 	} while( changed );
 }
 
@@ -884,7 +902,10 @@ main( int argc, char **argv )
 	int i, j;
 
 	if( im_init_world( argv[0] ) )
-	        error_exit( "unable to start VIPS" );
+	        error_exit( NULL );
+	textdomain( GETTEXT_PACKAGE );
+	setlocale( LC_ALL, "" );
+
 #ifdef DEBUG_FATAL
 	/* Set masks for debugging ... stop on any problem. 
 	 */
@@ -917,8 +938,7 @@ main( int argc, char **argv )
 
 	if( main_option_plugin ) {
 		if( !im_load_plugin( main_option_plugin ) )
-			error_exit( "unable to load plugin %s",
-				main_option_plugin );
+			error_exit( NULL ); 
 	}
 	if( main_option_cpph ) 
 		print_cppdecls( main_option_cpph );
@@ -930,7 +950,7 @@ main( int argc, char **argv )
 		print_list( main_option_list );
 	if( main_option_usage ) {
 		if( !(fn = im_find_function( main_option_usage )) )
-			error_exit( "unknown operation %s", main_option_usage );
+			error_exit( NULL );
 		usage( fn );
 	}
 	if( main_option_version ) 
@@ -968,24 +988,33 @@ main( int argc, char **argv )
 				name[strlen( name ) - 4] = '\0';
 
 			if( !(fn = im_find_function( name )) )
-				error_exit( "unknown function" );
+				error_exit( NULL );
 		}
 
 		/* Execute it!
 		 */
 		if( im_run_command( name, argc - 1, argv + 1 ) ) {
-			usage( fn );
-			error_exit( "error calling function" );
+			/* If there are no arguments and the operation failed,
+			 * show usage. There are no-arg operations, so we have
+			 * to try running it.
+			 */
+			if( argc == 1 )
+				usage( fn );
+			else
+				error_exit( NULL );
 		}
 	}
 	else if( argc > 1 ) {
 		/* Nope ... run the first arg instead.
 		 */
+		if( !(fn = im_find_function( argv[1] )) )
+			error_exit( NULL );
+
 		if( im_run_command( argv[1], argc - 2, argv + 2 ) ) {
-			if( !(fn = im_find_function( argv[1] )) )
-				error_exit( "unknown function" );
-			usage( fn );
-			error_exit( "error calling function" );
+			if( argc == 2 ) 
+				usage( fn );
+			else
+				error_exit( NULL );
 		}
 	}
 
